"use strict";

var _path = _interopRequireDefault(require("path"));

var _chalk = _interopRequireDefault(require("chalk"));

var _commander = _interopRequireDefault(require("commander"));

var _webpack = _interopRequireDefault(require("webpack"));

var _webpackNodeExternals = _interopRequireDefault(require("webpack-node-externals"));

var _child_process = require("child_process");

var _clearConsole = _interopRequireDefault(require("clear-console"));

var _forkTsCheckerWebpackPlugin = _interopRequireDefault(require("fork-ts-checker-webpack-plugin"));

var _package = _interopRequireDefault(require("../package.json"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _version$option$optio = new _commander.default.Command(_package.default.name).version(_package.default.version).option('--env <env>', 'environment').option('-e, --entry <entry-file>', 'entry file path').option('-o, --output <output-file>', 'output file path').option('-c, --ts-config <typescript-config-file>', 'tsconfig.json').option('--watch', 'watch mode', false).option('--verbose', 'verbose mode', false).option('--run', 'run compiled output', false).on('--help', function () {
  /* eslint-disable no-console */
  console.info();
  console.info('    If you have any problems, do not hesitate to file an issue:');
  console.info(`      ${_chalk.default.cyan(`https://github.com/vinpac/${_package.default.name}/issues/new`)}`);
  console.info();
  /* eslint-enable no-console */
}).parse(process.argv),
    entry = _version$option$optio.entry,
    output = _version$option$optio.output,
    _version$option$optio2 = _version$option$optio.watch,
    watchMode = _version$option$optio2 === void 0 ? false : _version$option$optio2,
    _version$option$optio3 = _version$option$optio.run,
    runMode = _version$option$optio3 === void 0 ? false : _version$option$optio3,
    _version$option$optio4 = _version$option$optio.verbose,
    verbose = _version$option$optio4 === void 0 ? false : _version$option$optio4,
    _version$option$optio5 = _version$option$optio.env,
    env = _version$option$optio5 === void 0 ? process.env.NODE_ENV || 'development' : _version$option$optio5,
    _version$option$optio6 = _version$option$optio.tsConfigFilepath,
    tsConfigFilepath = _version$option$optio6 === void 0 ? 'tsconfig.json' : _version$option$optio6;

var dev = env !== 'production';
var outputDir;
var outputFile = 'index.js';

if (!entry) {
  console.error('Missing entry');
  process.exit(1);
}

if (!output) {
  console.error('Missing output');
  process.exit(1);
}

if (output.endsWith('/')) {
  outputDir = output;
} else {
  var slashIndex = output.lastIndexOf('/');
  outputDir = output.substr(0, slashIndex);
  outputFile = output.substr(slashIndex + 1);
}

function loadTsConfig(filepath) {
  var json = require(_path.default.resolve(filepath));

  if (json.extends) {
    var baseJson = loadTsConfig(json.extends);
    json = Object.assign({}, baseJson, json);

    if (baseJson.compilerOptions) {
      json.compilerOptions = Object.assign({}, baseJson.compilerOptions, json.compilerOptions);
    }
  }

  return json;
}

var tsConfig = loadTsConfig(tsConfigFilepath);
var alias = {};

try {
  Object.keys(tsConfig.compilerOptions.paths).forEach(function (key) {
    var aliasKey = key;

    if (aliasKey.endsWith('/*')) {
      aliasKey = aliasKey.substr(0, aliasKey.length - 2);
    } else {
      aliasKey += '$';
    }

    tsConfig.compilerOptions.paths[key].forEach(function (target) {
      alias[aliasKey] = _path.default.resolve(tsConfig.compilerOptions.baseUrl || '', target.endsWith('/*') ? target.substr(0, target.length - 2) : target);
    });
  });
} catch (error) {
  console.info(error); // ...
}

var compiler = (0, _webpack.default)({
  target: 'node',
  mode: env,
  devtool: 'source-map',
  context: _path.default.resolve(),
  entry: _path.default.resolve(entry),
  output: {
    path: _path.default.resolve(outputDir),
    filename: outputFile,
    // Point sourcemap entries to original disk location (format as URL on Windows)
    devtoolModuleFilenameTemplate: function devtoolModuleFilenameTemplate(info) {
      return _path.default.resolve(info.absoluteResourcePath).replace(/\\/g, '/');
    }
  },
  module: {
    rules: [{
      test: /\.tsx?$/,
      exclude: /^node_modules/,
      loader: 'ts-loader',
      options: {
        silent: true,
        // disable type checker - we will use it in fork plugin
        transpileOnly: true,
        configFile: _path.default.resolve(tsConfigFilepath)
      }
    }]
  },
  plugins: [new _forkTsCheckerWebpackPlugin.default(), // Adds a banner to the top of each generated chunk
  // https://webpack.js.org/plugins/banner-plugin/
  new _webpack.default.BannerPlugin({
    banner: 'require("source-map-support").install();',
    raw: true,
    entryOnly: false
  })],
  bail: true,
  externals: [(0, _webpackNodeExternals.default)()],
  resolve: {
    extensions: ['.ts', '.tsx', '.js'],
    alias
  }
});
compiler.hooks.forkTsCheckerDone.tap('should', function (diagnostics, lints, elapsed) {
  var elapsedTime = Math.round(elapsed / 1000000);

  if (watchMode) {
    (0, _clearConsole.default)();
  }

  if (diagnostics.length) {
    console.info(_chalk.default.red(`${_chalk.default.bgRed.black(' FAIL ')} Compilation failed after ${elapsedTime}ms\n`));
    return;
  }

  if (runMode) {
    setTimeout(function () {
      console.info(_chalk.default.green(`${_chalk.default.bgGreen.black(' DONE ')} Successfully compiled server in ${elapsedTime}ms`));
      run();
    }, 1);
  }
});
var child;

var onChildExit = function onChildExit() {
  console.info(_chalk.default.red(`${_chalk.default.black.bgRed(' DONE ')} Process finished`));
};

var run = function run() {
  if (child) {
    child.removeListener('exit', onChildExit);
    child.kill('SIGTERM');
  }

  child = (0, _child_process.spawn)('node', [_path.default.join(outputDir, outputFile)], {
    env: Object.assign({
      NODE_ENV: env
    }, process.env),
    silent: false,
    stdio: 'inherit'
  });
  child.once('exit', onChildExit);
};

var onCompile = function onCompile(error, stats) {
  var failedMessage = _chalk.default.red(`${_chalk.default.bgRed.black(' FAIL ')} Compilation failed${stats ? ` after ${stats.endTime - stats.startTime}ms` : ''}`);

  if (error) {
    console.info(failedMessage);
    console.error(error);
    return;
  }

  if (stats.hasErrors()) {
    console.info(failedMessage);
    console.info(stats.toString({
      assets: verbose,
      builtAt: verbose,
      entrypoints: verbose,
      timings: false,
      cached: verbose,
      cachedAssets: verbose,
      chunks: verbose,
      chunkModules: verbose,
      colors: true,
      hash: verbose,
      modules: verbose,
      reasons: dev,
      version: verbose
    }));
  }
};

if (watchMode) {
  compiler.watch({}, onCompile);
} else {
  compiler.run(onCompile);
}